window.Gleam.initSearch([{"doc":"olive","title":"olive","content":"# olive 🫒\n\n[![Package Version](https://img.shields.io/hexpm/v/olive)](https://hex.pm/packages/olive)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/olive/)\n\n# Table of content\n- [Objectives](#objectives)\n  - [Who is this for?](#who-is-this-for)\n- [Caveats / Current limitations](#caveats--current-limitations)\n- [Installation and usage](#installation-and-usage)\n- [Example](#example)\n- [Logging](#logging)\n- [Technicalities](#technicalities)\n\n# Objectives\n\nOlive is a development tool to help working on a wisp server (AKA the _main server_ in this doc).\nOlive is a proxy to your _main server_, that adds the following features:\n\n- Hot reload of any modified erlang modules in your _main server_\n- Live reload of connected browsers to the olive proxy\n\n\nThe different pieces are:\n- A proxy to connect to instead of directly to the _main server_\n- Said proxy injects a live reload javascript snippet and handles a websocket connection to trigger connected browsers\n- Olive listens for file changes in your source code and:\n  1. Triggers a new build when code changes (`gleam build`)\n  2. Hot reload the modified erlang modules of your _main server_ (`erlang code:atomic_load/1`)\n  3. Sends a websocket message to connected browsers so they can reload\n\nAnything else sent to the proxy is directly rerouted to your _main server_.\n\n## Who is this for?\n- ❌ If you are using lustre, checkout lustre dev tools instead!\n- ❌ If you are developping an API server, you might be better off with a simple `watchexec`\n- ✅ If you are working on a server that renders HTML (vanilla, htmx or others), this might interest you!\n\n# Caveats / Current limitations\n\n&gt; Make sure to checkout the help for any configuration options with `gleam run -m olive -- --help`\n\nCurrent limitations are:\n\n- Your _main server_ needs to answer with a valid html document and a `&lt;/head&gt;` at some point.\n  Olive uses that to inject a piece of javascript for the websocket to connect.\n- Olive watches only for `.gleam` file changes under your project `src/` folder, as well as any path dependencies you might have.\n- Olive cannot reload any changes made in your main file where the `pub fn main()` is defined. See [Technicalities](#technicalities) for more info.\n\n\n# Installation and usage\n\nAdd  olive has a dev dependency of your _main server_:\n\n```sh\ngleam add --dev olive@1\n```\n\nThen, use gleam to run it:\n```sh\ngleam run -m olive\n```\n\nOlive takes care of running your _main server_ for you!\n\n# Example\n\nThis is a stripped example to show the relevant parts only. Be sure to check the docs for wisp / mist / any other lib that allows to have a server running on the BEAM.\n\nLet&#39;s say you did a `gleam new my_project`.\n\nIn `src/my_project.gleam`:\n```gleam\npub fn main() {\n  wisp.configure_logger()\n\n  let assert Ok(_) =\n    wisp_mist.handler(router.handle_request, &quot;secret_key&quot;)\n    |&gt; mist.new\n    |&gt; mist.port(3000)\n    |&gt; mist.start_http\n\n  process.sleep_forever()\n}\n```\nIn `src/my_project/router.gleam`:\n```gleam\npub fn handle_request(_req: Request) -&gt; Response {\n  response.new(200)\n  |&gt; response.set_body(mist.Bytes(bytes_tree.from_string(html(&quot;Hello world&quot;))))\n}\n\nfn html(content: String) {\n  &quot;&lt;html&gt;\n    &lt;head&gt;\n      &lt;title&gt;My project&lt;/title&gt;\n    &lt;/head&gt;\n    &lt;body&gt;&quot;\n    &lt;&gt; content\n    &lt;&gt; &quot;&lt;/body&gt;\n    &lt;/html&gt;&quot;\n}\n```\n\nNow, after installing `olive`, you can run `gleam run -m olive`, and the following will happen:\n1. Your server will be listening on localhost:3000\n2. The olive server will be listening on localhost:1234\n\nOpen localhost:1234, and you should be granted with a `Hello world`.\n\nNow update the file in `router.gleam` so the server sends back `Hello olive`, and voila!\nYour browser should refresh automatically after a quick rebuild and show you the new message 🎉\n\n\n# Logging\n\nOlive uses erlang `logger` module, and allows you to specify what goes out on your terminal.\n3 levels of filters are specified:\n- **All**: Will show all olive logs (the default).\n- **Error**: Will only show olive errors.\n- **None**: Will shutdown olive logs entirely.\n\nOlive logs 2 kind of log level: *Notice* and *Error*.\nIt also adds the metadata `{domain =&gt; [olive]}` to any logs so you can add your own handler on it if needed.\n\n&gt; ⚠️ I don&#39;t recommand turning off *Error* log level. ⚠️\n&gt;\n&gt; When rebuilding your project, any error from `gleam build` will be an Olive error log, and you won&#39;t\n&gt; see it if it is turned off!\n\n# Technicalities\n\nAfter building your project, Olive runs your program for you, similar to how `gleam run` would.\n\nIn technical words, it means Olive runs the following in erlang world:\n```erlang\nspawn_link(&quot;&lt;your_project&gt;@@main&quot;, run, [&quot;&lt;your_project&gt;&quot;]).\n```\nWith `your_project` being the `name` defined in `gleam.toml`.\n\n\nThe reload part lives also in erlang world, using this nice piece of code:\n```erlang\nreload_modules() -&gt;\n    Modules = code:modified_modules(),\n    lists:foreach(fun(Mod) -&gt; code:purge(Mod) end, Modules),\n    atomic_load(Modules).\n```\nThis purges any modified modules that a `gleam build` would have produced, then replaces them on the fly.\n\nNext time one of this module is called, the new code will take the lead.\nThis has one inconvenient, your main file never gets called more than once, so even if it gets replaced, it will always be the old code still running.\nMost of the time, the main file has a `process.sleep_forever()` so the old code stays.\n\nThis is why any changes to your main file will need a reboot of olive.\n","url":"index.html"},{"doc":"olive/logging","title":"LogLevel","content":"pub type LogLevel {\n  Emergency\n  Error\n  Notice\n}\n\nEmergency\n\n\nError\n\n\nNotice\n\n\nSynonyms:\nolive/logging.LogLevel\nolive/logging LogLevel","url":"olive/logging.html#LogLevel"},{"doc":"olive/logging","title":"LogLevelFilter","content":"pub type LogLevelFilter {\n  AllLogs\n  ErrorsOnly\n  NoLogs\n}\n\nAllLogs\n\n\nErrorsOnly\n\n\nNoLogs\n\n\nSynonyms:\nolive/logging.LogLevelFilter\nolive/logging LogLevelFilter","url":"olive/logging.html#LogLevelFilter"},{"doc":"olive/logging","title":"configure_logs","content":"pub fn configure_logs(level: LogLevelFilter) -&gt; Nil\n\nSynonyms:\nolive/logging.configure_logs\nolive/logging configure_logs","url":"olive/logging.html#configure_logs"},{"doc":"olive/logging","title":"error","content":"pub fn error(msg: String) -&gt; Nil\n\nSynonyms:\nolive/logging.error\nolive/logging error","url":"olive/logging.html#error"},{"doc":"olive/logging","title":"notice","content":"pub fn notice(msg: String) -&gt; Nil\n\nSynonyms:\nolive/logging.notice\nolive/logging notice","url":"olive/logging.html#notice"},{"doc":"olive/logging","title":"olive/logging","content":"","url":"olive/logging.html"},{"doc":"olive/cli","title":"CliOptions","content":"pub type CliOptions {\n  CliOptions(\n    main_port: Int,\n    proxy_port: Int,\n    bind: String,\n    log: logging.LogLevelFilter,\n  )\n}\n\nCliOptions(\n  main_port: Int,\n  proxy_port: Int,\n  bind: String,\n  log: logging.LogLevelFilter,\n)\n\n\nSynonyms:\nolive/cli.CliOptions\nolive/cli CliOptions","url":"olive/cli.html#CliOptions"},{"doc":"olive/cli","title":"get_options","content":"pub fn get_options() -&gt; Result(CliOptions, Nil)\n\nSynonyms:\nolive/cli.get_options\nolive/cli get_options","url":"olive/cli.html#get_options"},{"doc":"olive/cli","title":"olive/cli","content":" The `cli` module holds the list of options the user can pass when launching olive.","url":"olive/cli.html"},{"doc":"olive/client_registry","title":"ClientMessage","content":"pub type ClientMessage {\n  Reload\n}\n\nReload\n\n\nSynonyms:\nolive/client_registry.ClientMessage\nolive/client_registry ClientMessage","url":"olive/client_registry.html#ClientMessage"},{"doc":"olive/client_registry","title":"ClientRegistry","content":"pub opaque type ClientRegistry\n\n\nSynonyms:\nolive/client_registry.ClientRegistry\nolive/client_registry ClientRegistry","url":"olive/client_registry.html#ClientRegistry"},{"doc":"olive/client_registry","title":"add","content":"pub fn add(\n  registry: ClientRegistry,\n  client: Subject(ClientMessage),\n) -&gt; Nil\n\nSynonyms:\nolive/client_registry.add\nolive/client_registry add","url":"olive/client_registry.html#add"},{"doc":"olive/client_registry","title":"remove","content":"pub fn remove(\n  registry: ClientRegistry,\n  client: Subject(ClientMessage),\n) -&gt; Nil\n\nSynonyms:\nolive/client_registry.remove\nolive/client_registry remove","url":"olive/client_registry.html#remove"},{"doc":"olive/client_registry","title":"start","content":"pub fn start() -&gt; ClientRegistry\n\nSynonyms:\nolive/client_registry.start\nolive/client_registry start","url":"olive/client_registry.html#start"},{"doc":"olive/client_registry","title":"trigger","content":"pub fn trigger(registry: ClientRegistry) -&gt; Nil\n\nSynonyms:\nolive/client_registry.trigger\nolive/client_registry trigger","url":"olive/client_registry.html#trigger"},{"doc":"olive/client_registry","title":"olive/client_registry","content":" The `client_registry` module holds a list of connected clients (browsers)\n and allows to trigger those clients by sending them a `Reload` message.","url":"olive/client_registry.html"},{"doc":"olive/config","title":"Config","content":"pub type Config {\n  Config(\n    main_port: Int,\n    proxy_port: Int,\n    bind: String,\n    root: String,\n    name: String,\n    dirs: List(String),\n  )\n}\n Holds all the important bits to run olive\nConfig(\n  main_port: Int,\n  proxy_port: Int,\n  bind: String,\n  root: String,\n  name: String,\n  dirs: List(String),\n)\n\nmain_port\n&lt;p&gt;The main port needs to match the one listen by the main server&lt;/p&gt;\n\nproxy_port\n&lt;p&gt;The proxy port is used by olive to create the web proxy server&lt;/p&gt;\n\nbind\n&lt;p&gt;The bind address needs to match the one used by the main server\nand is used by the proxy to replicate the same behaviour&lt;/p&gt;\n\nroot\n&lt;p&gt;The root dir, where gleam.toml lives&lt;/p&gt;\n\nname\n&lt;p&gt;The name of the project (field “name” in gleam.toml)&lt;/p&gt;\n\ndirs\n&lt;p&gt;The list of dirs olive needs to watch to trigger live reload&lt;/p&gt;\n\nSynonyms:\nolive/config.Config\nolive/config Config","url":"olive/config.html#Config"},{"doc":"olive/config","title":"read_config","content":"pub fn read_config(options: CliOptions) -&gt; Result(Config, String)\n\nSynonyms:\nolive/config.read_config\nolive/config read_config","url":"olive/config.html#read_config"},{"doc":"olive/config","title":"olive/config","content":"","url":"olive/config.html"},{"doc":"olive/proxy","title":"start_http","content":"pub fn start_http(\n  config: Config,\n  clients: ClientRegistry,\n) -&gt; Result(Subject(Message), StartError)\n\nSynonyms:\nolive/proxy.start_http\nolive/proxy start_http","url":"olive/proxy.html#start_http"},{"doc":"olive/proxy","title":"olive/proxy","content":" The `proxy` module starts the proxy server that redirects to the &quot;main&quot; server\n It also injects a websocket connection and handles it, via the `client_registry`\n module, to add / remove them, and send them a websocket message to trigger the\n actual reload of the browser.","url":"olive/proxy.html"},{"doc":"olive/server_run","title":"reload_server_code","content":"pub fn reload_server_code(root: String) -&gt; Result(Nil, String)\n\nSynonyms:\nolive/server_run.reload_server_code\nolive/server_run reload_server_code","url":"olive/server_run.html#reload_server_code"},{"doc":"olive/server_run","title":"start_server","content":"pub fn start_server(\n  root: String,\n  name: String,\n) -&gt; Result(Pid, Atom)\n\nSynonyms:\nolive/server_run.start_server\nolive/server_run start_server","url":"olive/server_run.html#start_server"},{"doc":"olive/server_run","title":"olive/server_run","content":" the `server_run` module runs the &quot;main&quot; server and also handles the rebuilding\n of the main server code every time it is needed.","url":"olive/server_run.html"},{"doc":"olive/watcher","title":"InternalMsg","content":"pub opaque type InternalMsg\n\n\nSynonyms:\nolive/watcher.InternalMsg\nolive/watcher InternalMsg","url":"olive/watcher.html#InternalMsg"},{"doc":"olive/watcher","title":"Message","content":"pub type Message {\n  FilesChanged(String)\n}\n\nFilesChanged(String)\n\n\nSynonyms:\nolive/watcher.Message\nolive/watcher Message","url":"olive/watcher.html#Message"},{"doc":"olive/watcher","title":"start","content":"pub fn start(\n  config: Config,\n  watch_subject: Subject(Message),\n) -&gt; Result(Subject(InternalMsg), StartError)\n\nSynonyms:\nolive/watcher.start\nolive/watcher start","url":"olive/watcher.html#start"},{"doc":"olive/watcher","title":"olive/watcher","content":" The `watcher` module watches a list of dir and triggers a `FilesChanged` message\n to be handled by calling code.","url":"olive/watcher.html"},{"doc":"olive","title":"main","content":"pub fn main() -&gt; Nil\n Process is as follow:\n - Start a client registry\n - Start watcher, giving it a subject we will listen to.\n - Every time watcher sends us a message:\n   - Reload server code\n   - Send reload message to clients\nSynonyms:\nolive.main\nolive main","url":"olive.html#main"},{"doc":"olive","title":"olive","content":"","url":"olive.html"}]);